@@ -12,6 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+
 DEFAULT_OVSDB_CONNECTION = 'tcp:127.0.0.1:6640'
 DEFAULT_OVNNB_CONNECTION = 'tcp:127.0.0.1:6641'
 DEFAULT_TIMEOUT = 5
@@ -19,6 +20,8 @@ DEVICE_NAME_MAX_LEN = 14
 
 
 ACL_PRIORITY_MAX = 32767
+QOS_DSCP_MAX = 2 ** 6 - 1
+QOS_BANDWIDTH_MAX = 2 ** 32 - 1
 
 NAT_SNAT = 'snat'
 NAT_DNAT = 'dnat'
