@@ -40,6 +40,21 @@ class TestOvsdbIdl(base.FunctionalTestCase):
         cleanup_cmd = self.api.del_br(self.brname)
         self.addCleanup(cleanup_cmd.execute)
 
+    def test_idl_run_exception_terminates(self):
+        run = self.api.idl.run
+        with mock.patch.object(self.api.idl, "run") as runmock:
+            exceptions = iter([Exception("TestException")])
+
+            def side_effect():
+                try:
+                    raise next(exceptions)
+                except StopIteration:
+                    return run()
+
+            runmock.side_effect = side_effect
+            exists = self.api.br_exists(self.brname).execute(check_error=True)
+            self.assertFalse(exists)
+
     def test_br_exists_false(self):
         exists = self.api.br_exists(self.brname).execute(check_error=True)
         self.assertFalse(exists)
@@ -75,8 +90,8 @@ class TestOvsdbIdl(base.FunctionalTestCase):
     def _test_add_port(self):
         pname = utils.get_rand_device_name()
         with self.api.transaction(check_error=True) as txn:
-            txn.add(self.api.add_br(self.brname))
-            txn.add(self.api.add_port(self.brname, pname))
+            txn.extend([self.api.add_br(self.brname),
+                        self.api.add_port(self.brname, pname)])
         return pname
 
     def test_add_port(self):
@@ -118,6 +133,26 @@ class TestOvsdbIdl(base.FunctionalTestCase):
         self.addCleanup(delattr, type(connection), '_is_running')
         self.assertFalse(connection.stop(1))
 
+    def test_br_external_id(self):
+        KEY = "foo"
+        VALUE = "bar"
+        self.api.add_br(self.brname).execute(check_error=True)
+        self.api.br_set_external_id(self.brname, KEY, VALUE).execute(
+            check_error=True)
+        external_id = self.api.br_get_external_id(self.brname, KEY).execute(
+            check_error=True)
+        self.assertEqual(VALUE, external_id)
+
+    def test_iface_external_id(self):
+        KEY = "foo"
+        VALUE = "bar"
+        self.api.add_br(self.brname).execute(check_error=True)
+        self.api.iface_set_external_id(self.brname, KEY, VALUE).execute(
+            check_error=True)
+        external_id = self.api.iface_get_external_id(self.brname, KEY).execute(
+            check_error=True)
+        self.assertEqual(VALUE, external_id)
+
 
 class ImplIdlTestCase(base.FunctionalTestCase):
     schemas = ['Open_vSwitch']
@@ -162,5 +197,6 @@ class ImplIdlTestCase(base.FunctionalTestCase):
     def test_post_commit_vswitchd_incomplete_timeout(self, *args):
         # Due to timing issues we may rarely hit the global timeout, which
         # raises RuntimeError to match the vsctl implementation
-        self.api.ovsdb_connection.timeout = 1
+        mock.patch('ovsdbapp.backend.ovs_idl.transaction.'
+                   'Transaction.timeout_exceeded', return_value=True).start()
         self.assertRaises((exc.TimeoutException, RuntimeError), self._add_br)
