@@ -11,7 +11,6 @@
 #    under the License.
 
 import netaddr
-from oslo_utils import uuidutils
 import testscenarios
 
 from ovsdbapp.backend.ovs_idl import idlutils
@@ -490,7 +489,7 @@ class TestLspOps(OvnNorthboundTest):
     def test_lsp_set_addresses_invalid(self):
         self.assertRaises(
             TypeError,
-            self.api.lsp_set_addresses, 'fake', 'invalidaddress')
+            self.api.lsp_set_addresses, 'fake', ['invalidaddress'])
 
     def test_lsp_get_addresses(self):
         addresses = [
@@ -844,6 +843,32 @@ class TestLogicalRouterOps(OvnNorthboundTest):
                               if_exists=True)
         self.assertEqual(len(lr.nat), len(const.NAT_TYPES))
 
+    def test_lr_nat_del_specific_snat_ip_network(self):
+        lr = self._lr_add(utils.get_rand_device_name())
+        self._lr_nat_add(router=lr,
+                         nat_type=const.NAT_SNAT,
+                         logical_ip='10.17.4.0/24',
+                         external_ip='192.0.2.2')
+        # Attempt to delete NAT rule of type const.NAT_SNAT by passing
+        # an IP network (corresponding to logical_ip) as match_ip
+        self.api.lr_nat_del(lr.name,
+                            nat_type=const.NAT_SNAT,
+                            match_ip='10.17.4.0/24').execute(check_error=True)
+        # Assert that the NAT rule of type const.NAT_SNAT is deleted
+        self.assertEqual([], lr.nat)
+
+    def test_lr_nat_del_specific_snat_ip_network_not_found(self):
+        self.assertRaises(idlutils.RowNotFound, self._lr_nat_del,
+                          nat_type=const.NAT_SNAT, match_ip='10.17.4.0/24')
+
+    def test_lr_nat_del_specific_dnat_ip_network(self):
+        self.assertRaises(ValueError, self._lr_nat_del,
+                          nat_type=const.NAT_DNAT, match_ip='192.0.2.1/32')
+
+    def test_lr_nat_del_specific_both_ip_network(self):
+        self.assertRaises(ValueError, self._lr_nat_del,
+                          nat_type=const.NAT_BOTH, match_ip='192.0.2.0/24')
+
     def test_lr_nat_list(self):
         lr = self._three_nats()
         nats = self.api.lr_nat_list(lr.uuid).execute(check_error=True)
@@ -941,6 +966,15 @@ class TestLogicalRouterPortOps(OvnNorthboundTest):
         self.assertEqual(options, lrp.options)
         self.assertEqual(external_ids, lrp.external_ids)
 
+    def test_lrp_add_gw_chassis(self):
+        name, c1, c2 = [utils.get_rand_device_name() for _ in range(3)]
+        args = (name, 'de:ad:be:ef:4d:ad')
+        lrp = self._lrp_add(*args, gateway_chassis=(c1, c2))
+        c1 = self.api.lookup('Gateway_Chassis', "%s_%s" % (lrp.name, c1))
+        c2 = self.api.lookup('Gateway_Chassis', "%s_%s" % (lrp.name, c2))
+        self.assertIn(c1, lrp.gateway_chassis)
+        self.assertIn(c2, lrp.gateway_chassis)
+
     def test_lrp_del_uuid(self):
         lrp = self._lrp_add(None)
         self.api.lrp_del(lrp.uuid).execute(check_error=True)
@@ -998,6 +1032,14 @@ class TestLogicalRouterPortOps(OvnNorthboundTest):
         self.assertTrue(self.api.lrp_get_enabled(lrp.name).execute(
             check_error=True))
 
+    def test_lrp_get_set_options(self):
+        options = {'one': 'two', 'three': 'four'}
+        lrp = self._lrp_add(None)
+        self.api.lrp_set_options(lrp.uuid, **options).execute(
+            check_error=True)
+        self.assertEqual(options, self.api.lrp_get_options(lrp.uuid).execute(
+            check_error=True))
+
 
 class TestLoadBalancerOps(OvnNorthboundTest):
 
@@ -1343,7 +1385,7 @@ class TestPortGroup(OvnNorthboundTest):
     def setUp(self):
         super(TestPortGroup, self).setUp()
         self.switch = self.useFixture(fixtures.LogicalSwitchFixture()).obj
-        self.pg_name = 'testpg-%s' % uuidutils.generate_uuid()
+        self.pg_name = 'testpg-%s' % ovsdb_utils.generate_uuid()
 
     def test_port_group(self):
         # Assert the Port Group was added
@@ -1397,7 +1439,7 @@ class TestPortGroup(OvnNorthboundTest):
 
     def test_pg_del_ports_if_exists(self):
         self.api.pg_add(self.pg_name).execute(check_error=True)
-        non_existent_res = uuidutils.generate_uuid()
+        non_existent_res = ovsdb_utils.generate_uuid()
 
         # Assert that if if_exists is False (default) it will raise an error
         self.assertRaises(RuntimeError, self.api.pg_del_ports(self.pg_name,
@@ -1406,3 +1448,142 @@ class TestPortGroup(OvnNorthboundTest):
         # Assert that if if_exists is True it won't raise an error
         self.api.pg_del_ports(self.pg_name, non_existent_res,
                               if_exists=True).execute(check_error=True)
+
+
+class TestHAChassisGroup(OvnNorthboundTest):
+
+    def setUp(self):
+        super(TestHAChassisGroup, self).setUp()
+        self.hcg_name = 'ha-group-%s' % ovsdb_utils.generate_uuid()
+        self.chassis = 'chassis-%s' % ovsdb_utils.generate_uuid()
+
+    def test_ha_chassis_group(self):
+        # Assert the HA Chassis Group was added
+        self.api.ha_chassis_group_add(self.hcg_name).execute(check_error=True)
+        hcg = self.api.ha_chassis_group_get(self.hcg_name).execute(
+            check_error=True)
+        self.assertEqual(self.hcg_name, hcg.name)
+
+        # Assert the HA Chassis Group was deleted
+        self.api.ha_chassis_group_del(self.hcg_name).execute(check_error=True)
+        cmd = self.api.ha_chassis_group_get(self.hcg_name)
+        self.assertRaises(idlutils.RowNotFound, cmd.execute, check_error=True)
+
+    def test_ha_chassis_group_add_delete_chassis(self):
+        self.api.ha_chassis_group_add(self.hcg_name).execute(check_error=True)
+        priority = 20
+        self.api.ha_chassis_group_add_chassis(
+            self.hcg_name, self.chassis, priority).execute(check_error=True)
+
+        # Assert that the HA Chassis entry was created
+        row = self.api.db_find(
+            'HA_Chassis',
+            ('chassis_name', '=', self.chassis)).execute(check_error=True)
+        self.assertEqual(priority, row[0]['priority'])
+
+        # Assert that the HA Chassis entry was associated with
+        # the HA Chassis Group
+        hcg = self.api.ha_chassis_group_get(self.hcg_name).execute(
+            check_error=True)
+        self.assertEqual(self.chassis, hcg.ha_chassis[0].chassis_name)
+
+        # Deletes the HA Chassis entry
+        self.api.ha_chassis_group_del_chassis(
+            self.hcg_name, self.chassis).execute(check_error=True)
+        row = self.api.db_find(
+            'HA_Chassis',
+            ('chassis_name', '=', self.chassis)).execute(check_error=True)
+        self.assertEqual([], row)
+
+        # Assert that the deleted HA Chassis entry was dissociated from
+        # the HA Chassis Group
+        hcg = self.api.ha_chassis_group_get(self.hcg_name).execute(
+            check_error=True)
+        self.assertEqual([], hcg.ha_chassis)
+
+    def test_ha_chassis_group_if_exists(self):
+        self.api.ha_chassis_group_add(self.hcg_name).execute(check_error=True)
+        self.api.ha_chassis_group_add_chassis(
+            self.hcg_name, self.chassis, priority=10).execute(check_error=True)
+
+        # Deletes the HA Chassis entry
+        self.api.ha_chassis_group_del_chassis(
+            self.hcg_name, self.chassis).execute(check_error=True)
+        row = self.api.db_find(
+            'HA_Chassis',
+            ('chassis_name', '=', self.chassis)).execute(check_error=True)
+        self.assertEqual([], row)
+
+        # Tries to delete it again, since if_exists=True it shouldn't raise
+        # any errors
+        self.api.ha_chassis_group_del_chassis(
+            self.hcg_name, self.chassis, if_exists=True).execute(
+            check_error=True)
+
+        # Tries to delete it again with if_exists=False, now it should raise
+        # a RuntimeError
+        cmd = self.api.ha_chassis_group_del_chassis(
+            self.hcg_name, self.chassis, if_exists=False)
+        self.assertRaises(RuntimeError, cmd.execute, check_error=True)
+
+        # Deletes the HA Chassis Group entry
+        self.api.ha_chassis_group_del(self.hcg_name).execute(check_error=True)
+        cmd = self.api.ha_chassis_group_get(self.hcg_name)
+        self.assertRaises(idlutils.RowNotFound, cmd.execute, check_error=True)
+
+        # Tries to delete it again, since if_exists=True it shouldn't raise
+        # any errors
+        self.api.ha_chassis_group_del(
+            self.hcg_name, if_exists=True).execute(check_error=True)
+
+        # Tries to delete it again with if_exists=False, now it should raise
+        # a RuntimeError
+        cmd = self.api.ha_chassis_group_del(self.hcg_name)
+        self.assertRaises(RuntimeError, cmd.execute, check_error=True)
+
+    def test_ha_chassis_group_may_exist(self):
+        cmd = self.api.ha_chassis_group_add(self.hcg_name, may_exist=True)
+        hcg1 = cmd.execute(check_error=True)
+        hcg2 = cmd.execute(check_error=True)
+        self.assertEqual(hcg1, hcg2)
+
+
+class TestReferencedObjects(OvnNorthboundTest):
+    """Exercise adding a ls, lsp and lsp_address in a single transaction.
+
+    The main goal of this test is to make sure a transaction can use either
+    a name or an object notation in order to create an ls+lsp while in a
+    transaction context.
+    """
+    def setUp(self):
+        super(TestReferencedObjects, self).setUp()
+        self.ls_name = utils.get_rand_device_name()
+        self.lsp_name = utils.get_rand_device_name()
+        self.lsp_test_addresses = ['de:ad:be:ef:4d:ad 192.0.2.1']
+
+    def _check_values(self):
+        # Check: Make sure ls_get and lsp_get work (no RowNotFound exception)
+        self.api.ls_get(self.ls_name).execute(check_error=True)
+        self.api.lsp_get(self.lsp_name).execute(check_error=True)
+        self.assertEqual(self.lsp_test_addresses,
+                         self.api.lsp_get_addresses(self.lsp_name).execute(
+                             check_error=True))
+
+    def test_lsp_add_by_name(self):
+        with self.api.transaction(check_error=True) as txn:
+            txn.add(self.api.ls_add(self.ls_name))
+            txn.add(self.api.lsp_add(self.ls_name, self.lsp_name))
+            txn.add(self.api.lsp_set_addresses(self.lsp_name,
+                                               self.lsp_test_addresses))
+        self._check_values()
+
+    def test_lsp_add_by_object_via_db_create(self):
+        with self.api.transaction(check_error=True) as txn:
+            sw = txn.add(self.api.db_create_row('Logical_Switch',
+                                                name=self.ls_name))
+            prt = txn.add(self.api.db_create_row('Logical_Switch_Port',
+                                                 name=self.lsp_name))
+            txn.add(self.api.db_add('Logical_Switch', sw, "ports", prt))
+            txn.add(self.api.db_add('Logical_Switch_Port', prt,
+                                    "addresses", self.lsp_test_addresses[0]))
+        self._check_values()
